#!/bin/perl5.8

=head1 Name

SSM::Career --- Analyse occupational career

	Created: 2007-12-29
	Updated: 2007-12-30

=head1 Usage

use SSM::Career

=cut

package SSM::Career;

use Dredu;
use SSM::Occ95;

use Exporter ();
@ISA = qw(Exporter);
@EXPORT = qw( job8 job8_8595);
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

%jobmiss, %job8, %score75 are imported from SSM::Occ95

=cut

# ************************************
# Methods and Subroutines
# ************************************

=head1 Methods

=head2 init() - initialize the career object

=cut

sub new {
	my ($class, $data ) = @_;
	my $type = ref($class) || $class;
	my $self = [
		{
			_data  	=> $data,	# data object
		}
	]
	;
	bless $self, $type;
	$self -> init();
	return $self;
}


sub empty {
	my ($self) = @_;
	my $type = ref($self) || $self;
#	my $data = $self->[0]{_data} ;
	my $ret = [ () ];
	bless $ret, $type;
	$ret;
}

sub stage {
	my ( $self, @a ) = @_;
	my $type = ref($self) || $self;
	$ret = $self -> empty;
	foreach( @a ){
		push @$ret, $self->[$_];
	}
	$ret;
}

sub valid_range {
	my ($self) = @_;
	my $type = ref($self) || $self;
	my $ret = [ ] ;
	foreach( 0 .. $self->[0]->{_data}->Value('NStage') ){
		push @$ret, $self->[$_];
	}
	bless $ret, $type;
	$ret;
}

sub each {
	my ($self) = @_;
	my $type = ref($self) || $self;
	my @ret = () ;
	foreach( @$self ){
		my $new = [$_] ;
		bless $new, $type;
		push @ret, $new;
	}
	@ret;
}

sub first {
	my ($self) = @_;
	my $type = ref($self) || $self;
	my $ret = $self->empty;
	$$ret[0] = $self->[0]  if @$self;
	$ret;
}

sub last {
	my ($self) = @_;
	my $type = ref($self) || $self;
	my $ret = [ () ] ;
	$$ret[0] = $self->[-1]  if @$self;
	bless $ret, $type;
	$ret;
}

sub at_age {
	my ($self, $floor, $ceil ) = @_;
	my $type = ref($self) || $self;
		# new() expansion if $self is not a reference

	$ceil = $floor  unless defined $ceil;

	return $self->empty  if miss( $floor, $ceil );

	( $floor, $ceil ) = ( $ceil, $floor )  if $ceil<$floor;

	my @start = $self->start;
	push @start, $self->[0]->{_data}->Value('Age');
		# The last stage starts with the current age.

	my @range;

	my $i;
	for( $i=0; $i<$#start; ++$i ){
		if ( All_valid(@start[$i,$i+1]) && $start[$i]<=$ceil && $floor<=$start[$i+1] ){
			push @range , $i;
		}
	}

	return $self->empty  unless @range;

	my($min, $max);
	if( 1==@range ){
		$min = $max = $range[0];
	}
	else {
		@range = sort {$a<=>$b} @range;
		($min, $max) = @range[0,-1];
	}

	$ret = $self->empty;
	foreach( $min..$max ){
		push @$ret, $self->[$_];
	}
	return $ret;
}


sub page {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{page} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub number {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{number} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub industry {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{industry} } ;
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub size {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{size} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub status {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{status} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub job {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{job} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub post {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{post} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub start {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{start} };
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

	# -------- $career -> firstjob
	# ԒlF E̐Ei (1iځBAƌoȂ 0i)
	#	Õ[`
sub firstjob {
	my ($self) = @_;
	my $type = ref($self) || $self;
		# new() expansion if $self is not a reference.

	return $self->first   unless $self->[0]{_data} -> Value('NStage') ;

	foreach( $self->each ){
		return $_->empty  if miss( $_->status );

					# For 1985 and 1995 data: 8=student, 9=nonemployed
					# For 2005 data: 9=student, 10=nonemployed
		return $_  if $_->have_job ;
	}

	return $self->empty;
}

	# -------- int $self->job8595;
	# EiIuWFNg󂯎āAuǗvɊւEƃR[hϊĂ
	# w1995NSSMR[hEubNxp.114 ɂB
sub job8595 {
	my ($self) = @_;
	my @ret;
	my @pages = $self->each ;
	foreach $self ( @pages ){
		my($j) = $self->job;
		my($s) = $self->size;
		my($p) = $self->post;
		if( miss($j)	# l
			|| 501<=$j && $j<=544	# 
			|| 545<=$j && $j<=553	# Ǘ
			|| 593<=$j && $j<=598	# ۈ
			|| 608==$j           	# DEqCm
			|| 689<=$j && $j<=999	# EEs
			|| $s<=4   || miss($s)	# 29lȉ̖ԊƂl
		){
			push @ret, $j;
			next;
		}
		elsif( 10==$s ) {	# 
			if( 4<=$p && $p<=6 ) {	# ǗI
				push @ret, 545;
				next;
			}
			else{
				push @ret, $j ;
			}
		} else { 	# 30lȏ̖Ԋ
			if( 4==$p || 5==$p ){	# Вĉ̊ǗE (ے)
				push @ret, 550;
				next;
			}
			elsif( 6==$p ) {	# Ж (ВEdc))
				push @ret, 548;
				next;
			}
			else{
				push @ret, $j ;
			}
		}
	}

	warn( "Number mismatch: @pages -> @ret.\n" )  if @pages != @ret;

	1==@ret ? $ret[0] : @ret;
}

sub class_job8_8595 {
	my ($self) = @_;
	my @ret = $self->job8595;
	foreach(@ret){
		if( exists $jobmiss{$_} ){
			$_=Missval($jobmiss{$_})
		}
		elsif(valid($_)){
			$_ = $job8{$_};
		}
	}
print STDERR join( ';', $self->job8595), @ret  if grep {!$_} @ret;
	1==@ret ? $ret[0] : @ret;
}

sub class_job8 {
	my ($self) = @_;
	my @ret = $self->job;
	foreach(@ret){
		if( exists $jobmiss{$_} ){
			$_=Missval($jobmiss{$_})
		}
		elsif(valid($_)){
			$_ = $job8{$_};
		}
	}
	1==@ret ? $ret[0] : @ret;
}

=head2 firstjob_single

	# ԒlF OE̐Ei (qoYOȂ炻D)
	#     Oɂ͏AƌoȂ->0i
	# ҂ɂĂ͏Ê܂ܕԂB
	# ̒n (MarStat) 85N95N 3,4 (ʁE)ւĂB
	# 	ł 1 () ǂɂ̂ŁAʃ[`őΏłB
	# Age_marriage ́A85Nł͏, 95Nł݂͌̌łB

=cut

sub firstjob_single {
	my ($self) = @_;
	my $type = ref($self) || $self;
	my $fj = $self -> firstjob;

	return $fj
		if ! @$fj  
		|| 1 == $self->[0]{_data}->Value('MarStat')
		|| 0 == $self->[0]{_data}->Value('NStage')
	;
		# missing, no experience or ummarried

	my $age_m = $self->[0]{_data} -> Age_marriage;
	return $self->empty  if miss($age_m);
	my $fjs = $fj->start;
	return $self->empty  if miss($fjs);

	my $age_fc = $self->[0]{_data} -> Age_first_birth;
	if(  0<$age_fc  && $age_fc<$age_m ) { 
		return $fj  if $fjs < $age_fc ;
	} else {
		return $fj	if $fjs < $age_m ;
	}

	$self -> first;
}


=head2 $career->class_new8( int $x ) - SSM new occupational category (8)
	
	Argument: $x is the maximum for small firm 
	Return value: array of the following values
		1	
		2	ƊǗEE̔
		3	ƊǗEE̔
		4	cƊǗEE̔
		5	Ƃa
		6	Ƃa
		7	cƂa
		8	_

=cut

sub class_new8 {
	my ($self, $x) = @_;
	$x ||= 7;
	my @ret=();
	foreach( $self->each ){
		push @ret, $_->_class_new8( $x );
	}
	return Missval('GHOST')  unless @ret;
	1==@ret ? $ret[0] : @ret;
}

sub _class_new8 {
	my ($self, $t) = @_;
	my @ret=();
	my $e = $self->class_status4;
	my $s = $self->size;
	my $o8= $self->class_job8_8595;
		# ǗEῗȂĂ悢Ƃ: occ85.pl

	return $o8	if miss($o8)  ;	# Missing
	return $o8	if 1==$o8 || 8==$o8  ;	# E_

	if( 2<=$o8 && $o8<=4 ) {	# ǗEE̔
		return 4	if  1==$e ;	# v
		return $s	if miss($s) ;	# K͌
		return 2	if  $t<$s ;	# v
		return 3		# v
	}

	if( 5<=$o8 && $o8<=7) { 	# u[J[
		return 7	if  1==$e ;	# a
		return $s	if miss($s) ;	# K͌
		return 5	if  $t<$s ;	# a
		return 6		# a
	}

print STDERR join(':', $self->page, $self->status, $e, $o8, $s );
	Missval('BLANK');	# ̑͌l
}


# ******************************************
1;  # End
# ******************************************


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2007 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
