#!/bin/perl5.8

=head1 Name

SSM::SSM1995 --- Subroutines for SSM 2005 data

	Created: 2007-12-16
	Updated: 2007-12-16
	Updated: 2011-01-30

=head1 Usage

use SSM::SSM1995

=cut

package SSM::SSM1995;

use Dredu;

use Exporter ();
@ISA = qw(Dredu Exporter);
@EXPORT = qw( Eduyear Income_10000yen %eduyear Id %income_10000yen);
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

=head2 %kind

=cut

%kind = (
	1 => 'A',
	2 => 'B',
	3 => 'P',
);

=head2 %eduyear

=cut

%eduyear = (
	 1,	 6,		# (old) standard elementary
	 2,	 8,		# (old) high elementary
	 3,	11,		# (old) middle school
	 4,	11,		# (old) middle school (Zitugyou)
	 5,	11,		# (old) middle school (Sihan)
	 6,	14,		# (old) high school
	 7,	17,		# (old) university
	12,	 9,		# (new) junior-high
	13,	12,		# (new) high schoool
	14,	14,		# (new) junior college
	15,	16,		# (new) university
	16,	18,		# (new) Graduate school
) ;     
          
# ************************************
# Methods
# ************************************

=head1 Exported Subroutines

=head2 init() - initialize

=cut

sub init {
}

=head2 Kind() - Kind of the Questionnaire

	return: 1(A) / 2(B) / 3(P)

=cut

sub Kind {
	my ($self) = shift;
	return $self->Value( 'KIND1' )   if $self;
	return $main::KIND1;
}

=head2 Kind_p() - Prefix for each questionnaire

	return: A / B / P(restige)

=cut


sub Kind_p {
	my($self) = shift;
	my $k ;
	if( $self ){
		$k = $self->Value( 'KIND1' ) 
	}
	else{
		$k = $main::KIND1 ;
	}
	return $kind{$k}  if exists $kind{$k};
	warn( "Invalid questionnaire: $k." );
}


=head2 Id() - Identification Number

=cut

sub Id {
	my ($self) = shift;
	return sprintf '%d%02d%03d' , $self->values( qw/KIND1 IDT1 IDP1/ )   if $self;
	return sprintf '%d%02d%03d' , $main::KIND1, $main::IDT1, $main::IDP1;
}


=head2 Age_last_birth() - how old was the respondent at the last childbirth

=cut


sub Age_marriage {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	return $self->Value('MarAge');
}

	# -------- int &years_old( int x );
	# ԒlFx ` x+1 NO̖{lN
	#	95Nf[^̒̒qNȂǂƖ{lNƂ̍ƂB
	#	{l̒a𗘗pĐxBNB
	# &age_fc, &age_lc Ăяo
sub years_old {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	foreach(@a){
		next if miss($_);
		$_ = ${ $self->{var}{Age} } - $_ ;	# {lNƂ̍Ƃ
		next if $_== $self-> Age_marriage ;	# NƓȂ炻̂܂ܕԂB
		next unless 5<=${ $self->{var}{Q1_2M} } && ${ $self->{var}{Q1_2M} }<=10 ;
		--$_ ;	# aɂ͌l͂Ȃ͂B5`10Ȃ1Ђ
	}
	return $a[0] if 1==@a;
	@a;
}


sub children_age {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);

		# no child
	return ()  unless ${ $self->{var}{Q29_1} };

	my @child_age = ( ${ $self->{var}{Q29_2} } ) ;
	$child_age[1] =  ${ $self->{var}{Q29_3} }  if 1 < ${ $self->{var}{Q29_1} }  && valid(${ $self->{var}{Q29_1} });

	@child_age;
}

sub age_births {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);

	my @c = $self->children_age;
	$self->years_old(@c);
}

sub Age_first_birth {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my @a = $self->age_births;
	return Missval('OUT')  unless @a;
	@a[0];
}

sub Age_last_birth {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my @a = $self->age_births;
	return Missval('OUT')  unless @a;
	@a[-1];
}

=head2 household() - Return the array for household members (Q31_1, Q31_201..Q31_225)

	Return[0]    : number of the household members (=Q31_1 for Questionnaire A )
	Return[1..25]: 0 or 1 for each relation

=cut

sub household {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $data.")  unless ref($self);

	my $k = $self->Kind;
	return ( ( Missval('BLANK') )x 26 ) if 3==$k ;

	my $n = (1==$k) ? 31 : 21 ;
	my @r =( $self -> Value( "Q${n}_1" ) );
	my $q = 1;
	foreach $q(1..25) {
		$r[$q] = $self -> Value( sprintf "Q${n}_2%02d" , $q );
	}
	@r;
}


=head2 income() - income of the household, respondent, and the spouse

	return: array for income( Household, Personal, Spuse)

=cut

sub income {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	return $self->values( 'Q45_3', 'Q45_1', 'Q45_2' ) if 1==$self->Kind;
	return $self->values( 'Q44_3', 'Q44_1', 'Q44_2' ) if 2==$self->Kind;
	return $self->values( 'Q14_3', 'Q14_1', 'Q14_2' ) if 3==$self->Kind;
	warn("Invalid KIND of questionnaire.")
}

=head2 Income_10000yen($x) - Metric of income (x 10,000 yen)

	$x: SSM 1995 income code

=cut

%income_10000yen =(
	  1=> 0,
	  2=> (70)/2,
	  3=> 100,
	  4=> 200,
	  5=> 300,
	  6=> 400,
	  7=> 500,
	  8=> 600,
	  9=> 700,
	 10=> 800,
	 11=> 900,
	 12=>1000 ,
	 13=>1300 ,
	 14=>1500,
	 15=>1700,
	 16=>2000,
	 17=>(2300+2800)/2,
	 18=>(2800+3300)/2,
	 19=>(3300+3800)/2,
	 20=>(3800+4300)/2,
	 21=>(4300+4800)/2,
	 22=>(4800+5300)/2,
	 23=>(5300+5800)/2,
	 24=>(5800+6300)/2,
	 25=>(6300+6800)/2,
	 26=>(6800+7300)/2,
	 27=>(7300+7800)/2,
	 28=>(7800+8300)/2,
	 29=>(8300+8800)/2,
	 30=>(8800+9300)/2,
	 31=>(9300+9800)/2,
	 32=>(9800+10300)/2,
);


# To convert income
sub Income_10000yen {
	my $x = @_[0];
	return $x if miss($x);

	if( exists $income_10000yen{$x} ){
		return $income_10000yen{$x};
	}
	else {
		warn( "Illegal value for income: $x\n" );
		Missval( 'BLANK' );
	}
}


# ************************************
# Subroutines
# ************************************

=head2 Eduyear($e) - Return duration of standard school education

	$e: SSM 1995 education code

=cut

sub Eduyear {
	return $_[0] if miss($_[0]) ;
	return $eduyear{$_[0]} if exists $eduyear{$_[0]} ;
	warn ( "Invalid education code: $_[0]\n" );
	undef;
}

=head2 Eduyear($e) - Return duration of standard school education

	$e: SSM 1995 education code

=cut

sub Eduyear {
	return $_[0] if miss($_[0]) ;
	return $eduyear{$_[0]} if exists $eduyear{$_[0]} ;
	warn ( "Invalid education code: $_[0]\n" );
	undef;
}

1;


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2007 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
