#!/bin/perl5.8

=head1 Name

SSM::SSM1995::Career --- Analyse occupational career

	Created: 2007-12-16
	Updated: 2007-12-29
	Updated: 2007-12-30

=head1 Usage

use SSM::SSM1995::Career

=cut

package SSM::SSM1995::Career;

use Dredu;
use SSM::SSM1995;
use SSM::Occ95;
use SSM::Career;

use Exporter ();
@ISA = qw(SSM::Career Exporter);
@EXPORT = qw( );
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

%jobmiss, %job8, %score75 are imported from SSM::Occ95

=cut

    # Hash table for four categories of employment status
    # Used by class_status4()
%status4 = (
     1, 1,  # oc
     2, 2,  # 
     3, 3,  # Վ
     4, 3,  # h
     5, 1,  # c
     6, 1,  # Ƒ
     7, 3,  # E
     8, 9,  # w (Q23 E)
     9, 9,  # E (Q23 s)
    18, 9,  # AƌoȂ
) ;


# ************************************
# Methods and Subroutines
# ************************************

=head1 Methods

Constructor new() is inherited from SSM::Career

=head2 init() - initialize the career object

=cut

sub init {
	my ($self) = @_;
	my $data = $self->[0]{_data} ;
	my %work = (
		'Q7P%02d' , 'page' ,
		'Q7W%02d', 'number' ,
		'Q7A%02d' , 'status' ,
		'Q7C%02d' , 'industry' ,
		'Q7D%02d' , 'size' ,
		'Q7E%02d' , 'job' ,
		'Q7F%02d' , 'post' ,
		'Q7G%02d', 'start' ,
	) ;

	my $n;
	foreach $n ( 1..17 ){
		foreach( keys %work ) {
			my $q = sprintf( $_ , $n ) ;
			$self->[$n]{$work{$_}} = $data->{var}{$q};
			$self->[$n]{_data} = $data;
		}
	}
	
	my $zero = 0;
	my $ten =  9;		# E
	my $out  = Missval('OUT');
	my $ghost  = Missval('GHOST');
	$self->[0]{ page  }  = \$zero ;
	$self->[0]{ number } = \$zero ;
	$self->[0]{ industry} =\$out ;
	$self->[0]{ size   } = \$out ;
	$self->[0]{ status } = \$ten ;
	$self->[0]{ job   } =  \$out ;
	$self->[0]{ post   } = \$out ;
	$self->[0]{ start  } = \$zero ;
}

=head2 current_job() - initialize the current job

=cut
sub current_job {
	my ($t, $data, $k) = @_;
	my $self = $t->empty;
	$self->[0]{_data} =$data;

	my $q = 4;
	$q = 9  if $k eq 'p' || 3==$k;
	$self -> assign($q);
}

sub assign {
	my ($self, $q) = @_;
	my $data = $self->[0]{_data};
	my $zero = 0;
	$self->[0]{ page  }  = \$zero ;
	$self->[0]{ number } = \$zero ;
	$self->[0]{ industry} =$data->{var}{"Q${q}C"} ;
	$self->[0]{ size   } = $data->{var}{"Q${q}D"} ;
	$self->[0]{ status } = $data->{var}{"Q${q}A"} ;
	$self->[0]{ job   } =  $data->{var}{"Q${q}E"} ;
	$self->[0]{ post   } = $data->{var}{"Q${q}F"} ;
	$self;
}

=head2 have_job() - Stages having a job

Return value: array of status for stages having a job

=cut

sub have_job {
	grep { $_<8 } valid( $_[0] -> status );
}


    # Convert employment status into four categories
sub class_status4 {
	my ($self) = @_;
	return Missval('GHOST')  unless $self->each;
	my $x;
	my @ret;
	foreach $x ( $self->each ){
		my $j = $x->class_job8_8595;
		my $s = $x->status;
		if( 8 == $j ) {   push @ret, 1 ;  } # Agricultural work
		elsif(miss($s)) { push @ret, $s ; } # Missing status
		elsif(exists $status4{$s}) { push @ret, $status4{$s} ; } # Convert by hash
		else { push @ret, undef ; }            # Else undef
	}
	1==@ret ? $ret[0] : @ret;
}


# ***************************************************
1;
# ***************************************************


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2007 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
