#!/bin/perl5.8

=head1 Name

SSM::SSM2005 --- Subroutines for SSM 2005 data

	Created: 2007-09-06
	Updated: 2007-11-09

=head1 Usage

use SSM::SSM2005

=cut

package SSM::SSM2005;

use Dredu;

use Exporter ();
@ISA = qw(Dredu Exporter);
@EXPORT = qw( Eduyear %eduyear Id Income_10000yen %income_10000yen);
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

=head2 %eduyear

=cut


# ************************************
# Methods
# ************************************

=head1 Exported Subroutines

=head2 init() - initialize

=cut

sub init {
	%eduyear = (
		1,	 9,
		2,	12,
		3,	14,
		4,	14,
		5,	16,
		6,	18,
		7,	Missval('NA'),
		8,	0,
		miss_miss()
	) ;
}

=head2 Id() - Identification Number

=cut


sub Id {
	my($self) = shift;
#	return sprintf '%02d%03d%02d' , $self->values( 'NUMS' , 'NUMP' , 'NUMI' ) if $self;
	return sprintf '%02d%03d%02d' , $main::NUMS, $main::NUMP, $main::NUMI;
}



=head2 Age_last_birth() - how old was the respondent at the last childbirth

=cut


sub Age_marriage {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my $marstat = $self->Value('MarStat');
	return $self->Value('Q24_2AM') if 1==$marstat || 2==$marstat ;
	return $self->Value('Q24_2BD') if 3==$marstat || 4==$marstat ;
	my @v = valid( $self->Value('Q24_2AM'), $self->Value('Q24_2BD') );
	warn("Marital Status is $marstat.\n") if valid($marstat);
	@v ? $v[0] : $marstat ;
}

	# -------- int &years_old( int x );
	# ԒlFx ` x+1 NO̖{lN
	#	95Nf[^̒̒qNȂǂƖ{lNƂ̍ƂB
	#	{l̒a𗘗pĐxBNB
	# &age_fc, &age_lc Ăяo
sub years_old {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	foreach(@a){
		next if miss($_);
		$_ = ${ $self->{var}{Age} } - $_ ;	# {lNƂ̍Ƃ
		next if $_== $self-> Age_marriage ;	# NƓȂ炻̂܂ܕԂB
		next unless 5<=${ $self->{var}{Q01_2M} } && ${ $self->{var}{Q01_2M} }<=10 ;
		--$_ ;	# aɂ͌l͂Ȃ͂B5`10Ȃ1Ђ
	}
	return $a[0] if 1==@a;
	@a;
}


sub children_age {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);

		# no child
	return ()  unless ${ $self->{var}{Q28_1} };

	my @child_age =();
	push @child_age,
		${ $self->{var}{Q28_2A1} },
		${ $self->{var}{Q28_2A2} },
		${ $self->{var}{Q28_2A3} },
		${ $self->{var}{Q28_2A4} },
		${ $self->{var}{Q28_2A5} },
		${ $self->{var}{Q28_2A6} },
		${ $self->{var}{Q28_2A7} }
	;

	pop @child_age while( $child_age[-1] eq Missval('OUT') ) ;
	@child_age;
}

sub age_births {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);

	my @c = $self->children_age;
	$self->years_old(@c);
}

sub Age_first_birth {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my @a = $self->age_births;
	return Missval('OUT')  unless @a;
	@a[0];
}

sub Age_last_birth {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my @a = $self->age_births;
	return Missval('OUT')  unless @a;
	@a[-1];
}

=head2 household_member() - List of coresident household member

Return: list of (0,1).
The first element is dummy so that the index correspond to the question-choice number.

=cut

sub household_member {
	my ( $self ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my @household_member = ();
	foreach( 1..19 ){
		$household_member[$_] = Common_question( 
			$self->Value( sprintf "A20_2_%02d", $_ ),
			$self->Value( sprintf "B07_2_%02d", $_ ),
		);
	}
	@household_member = ( Missval('NA') ) x 20
		unless  grep { 2 != $_ } @household_member[1..19]
	;
	foreach(@household_member){
		$_ = 0  if 2==$_;
	}
	@household_member;
}


# ************************************
# Subroutines
# ************************************

=head2 Eduyear($e) - Return duration of standard school education

	$e: SSM education code

=cut

sub Eduyear {
	return $eduyear{$_[0]} if defined $eduyear{$_[0]} ;
	warn ( "Invalid education code: $_[0]\n" );
	undef;
}


=head2 Income_10000yen($x) - Convert income

	$x: SSM income code

=cut

%income_10000yen =(
	  1=> 0,
	  2=> (25)/2,
	  3=> (25+50)/2,
	  4=> (50+75)/2,
	  5=> (75+100)/2,
	  6=> (100+125)/2,
	  7=> (125+150)/2,
	  8=> (150+200)/2,
	  9=> (200+250)/2,
	 10=> (250+300)/2,
	 11=> (300+350)/2,
	 12=> (350+400)/2,
	 13=> (400+550)/2,
	 14=> 500,
	 15=> 600,
	 16=> 700,
	 17=> 800,
	 18=> 900,
	 19=>1000,
	 20=>1100,
	 21=>1200,
	 22=>1300,
	 23=>1400,
	 24=>1500,
	 25=>1600,
	 26=>1700,
	 27=>1800,
	 28=>1900,
	 29=>2000,
	 30=>2100,
);


# To convert income
sub Income_10000yen {
	my $x = @_[0];
	return $x if miss($x);
	
	if( exists $income_10000yen{$x} ){
		return $income_10000yen{$x};
	}
	else {
		warn( "Illegal value for income: $x\n" );
		Missval( 'BLANK' );
	}
}




1;


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2007,2011 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
