#!/bin/perl5.8

=head1 Name

SSM::SSM2005::Career --- Analyse occupational career

	Created: 2007-09-06
	Updated: 2007-12-30

=head1 Usage

use SSM::SSM2005::Career

=cut

package SSM::SSM2005::Career;

use Dredu;
use SSM::SSM2005;
use SSM::Occ95;
use SSM::Career;

use Exporter ();
@ISA = qw(SSM::Career Exporter);
@EXPORT = qw( );
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

%jobmiss, %job8, %score75 are imported from SSM::Occ95

=cut

    # Hash table for four categories of employment status
    # Used by class_status4()
%status4 = (
     1, 1,  # oc
     2, 2,  # 
     3, 3,  # Վ
     4, 3,  # h
     5, 3,  # _
     6, 1,  # c
     7, 1,  # Ƒ
     8, 3,  # E
     9, 9,  # w (Q23 E)
    10, 9,  # E (Q23 s)
    11, 9,  # s
#   18, 9,  # AƌoȂ
) ;

# ************************************
# Methods and Subroutines
# ************************************

=head1 Methods

Constructor new() is inherited from SSM::Career

=head2 init() - initialize the career object

=cut


sub init {
	my ($self) = @_;
	my $data = $self->[0]{_data} ;
	my %work = (
		'NUM%02d' , 'page' ,
		'Q08N%02d', 'number' ,
		'Q08A%02d', 'reason' ,	# Q08A01 is not exist
		'Q08A%02d_7','reason7' ,	# Q08A01_7 is not exist
		'Q08B%02d' , 'industry' ,
		'Q08B%02dI', 'isic' ,
		'Q08C%02d' , 'size' ,
		'Q08D%02d' , 'status' ,
		'Q08E%02d' , 'job' ,
		'Q08E%02dI', 'isco' ,
		'Q08F%02d' , 'post' ,
		'Q08PA%02d', 'start' ,
		'Q08G%02d' , 'income' 	# Q08G01 is not exist
	) ;

	my $n;
	foreach $n ( 1..16 ){
		foreach( keys %work ) {
			my $q = sprintf( $_ , $n ) ;
			$self->[$n]{$work{$_}} = $data->{var}{$q};
			$self->[$n]{_data} = $data;
		}
	}
	
	my $zero = 0;
	my $ten = 10;
	my $out  = Missval('OUT');
	my $ghost  = Missval('GHOST');

	$self->[0]{ page  }  = \$zero ;
	$self->[0]{ number } = \$zero ;
	$self->[0]{ reason } = \$out ;
	$self->[0]{ reason7 } =\$out ;
	$self->[0]{ industry} =\$out ;
	$self->[0]{ isic   } = \$out ;
	$self->[0]{ size   } = \$out ;
	$self->[0]{ status } = \$ten ;
	$self->[0]{ job   } =  \$out ;
	$self->[0]{ isco   } = \$out ;
	$self->[0]{ post   } = \$out ;
	$self->[0]{ start  } = \$zero ;
	$self->[0]{ income } = \$out ;

	$self->[1]{ reason } = \$ghost ;	# Q08G01 is not exist
	$self->[1]{ reason7 } =\$ghost ;	# Q08A01_7 is not exist
	$self->[1]{ income } = \$ghost ;	# Q08A01 is not exist
}


sub reason {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{reason} };
	}
	1==@ret ? $ret[0] : @ret;
}

sub reason7 {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{reason7} };
	}

	1==@ret ? $ret[0] : @ret;
}


sub isic {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{isic} };
	}
	1==@ret ? $ret[0] : @ret;
}


sub isco {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{isco} };
	}

	1==@ret ? $ret[0] : @ret;
}


sub income {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( @$self ){
		push @ret , ${ $x->{income} };
	}
	1==@ret ? $ret[0] : @ret;
}

=head2 have_job() - Stages having a job

Return value: array of status for stages having a job

=cut

sub have_job {
	grep { $_ < 9 } valid( $_[0] -> status );
}



    # Convert employment status into four categories
sub class_status4 {
	my ($self) = @_;
	my $x;
	my @ret;
	foreach $x ( $self->each ){
		my $j = $x->class_job8_8595;
		my $s = $x->status;
		if( 8 == $j ) {   push @ret, 1 ;  } # _Ƃ͎c
		elsif(miss($s)) { push @ret, $s ; } # Missing status
		elsif(exists $status4{$s}) { push @ret, $status4{$s} ; } # Convert by hash
		else { push @ret, undef ; }            # Else undef
	}
	return Missval('GHOST') unless @ret;
	1==@ret ? $ret[0] : @ret;
}


# ********************************
1;
# ********************************


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2007 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
